
#include <fstream>
#include <vector>
#include <cmath>
#include <iostream>

const double eps = 1e-10;
const double inf = 1e10;

double f_value(const std::vector<double>& c, double x)
{
  double res = 0;

  for (std::vector<double>::const_reverse_iterator it = c.rbegin(); it != c.rend(); ++it)
    res = res * x + *it;

  return res;
}

double find_r (const std::vector<double>& c, double lb, double ub)
{
  while (ub - lb > eps) 
  {
    if (std::abs(f_value(c, lb)) < eps) return lb;
    if (std::abs(f_value(c, ub)) < eps) return ub;

    double md = (ub + lb) / 2;

    if (f_value(c, lb)*f_value(c, md) < 0) 
      ub = md;
    else
      lb = md;
  }

  return ub;
}

int main (int argc, char** argv)
{
  std::vector<std::vector<double> > X(3), b(3), c(3);
  std::vector<double> m(3), e(3), h(4), res(3);
  int n, null_col, kmin;
  double dmin, ak, bk, ck, dk, k1, k2;

  for (int i = 0; i < 3; ++i) { b[i].resize(3); c[i].resize(3); }

  std::fstream input("input.txt", std::ios_base::in);
  std::fstream output("output.txt", std::ios_base::out);

  input >> n;
  for (int i = 0; i < 3; ++i) X[i].resize(n);
  for (int i = 0; i < n; ++i)
    for (int j = 0; j < 3; ++j)
    {
      input >> X[j][i];
      m[j] += X[j][i];
    }

  for (int i = 0; i < 3; ++i)
    for (int j = 0; j < 3; ++j)
    {
      for (int k = 0; k < n; ++k)
        b[i][j] += (X[i][k] - m[i] / n) * (X[j][k] - m[j] / n);
      b[i][j] /= n;
    }

  h[3] = -1; h[2] = b[0][0] + b[1][1] + b[2][2];
  h[1] = -b[1][1]*b[2][2] - b[0][0]*b[2][2] - b[0][0]*b[1][1] + b[1][2]*b[1][2] + b[0][1]*b[0][1] + b[0][2]*b[0][2];
  h[0] = b[0][0]*b[1][1]*b[2][2] - b[0][0]*b[1][2]*b[2][1] - b[0][1]*b[1][0]*b[2][2] + b[0][1]*b[2][0]*b[1][2] + b[0][2]*b[1][0]*b[2][1] - b[0][2]*b[1][1]*b[2][0];

  ak = 3*h[3]; bk = 2*h[2]; ck = h[1];
  dk = bk*bk - 4*ak*ck;
  k1 = (-bk - sqrt(dk)) / (2*ak);
  k2 = (-bk + sqrt(dk)) / (2*ak);

  if (k1 > k2) { k1 = k1 + k2; k2 = k1 - k2; k1 = k1 - k2; }

  e[0] = find_r(h, -inf, k1);
  e[1] = find_r(h, k1, k2);
  e[2] = find_r(h, k2, inf);

  dmin = inf;
  kmin = -1;
  for (int i = 0; i < 3; ++i)
    for (int j = i + 1; j < 3; ++j)
      if (std::abs(e[i] - e[j]) < dmin)
      {
        dmin = std::abs(e[i] - e[j]);
        kmin = 3 - i - j;
      }

  for (int i = 0; i < 3; ++i)
  {
    for (int j = 0; j < 3; ++j) c[i][j] = b[i][j];
    c[i][i] -= e[kmin];
  }

  null_col = -1;
  for (int i = 0; i < 3; ++i)
  {
    bool only_null = true;
    for (int j = 0; j < 3; ++j) only_null &= (std::abs(c[j][i]) < eps);
    if (only_null) null_col = i;
  }

  if (null_col != -1)
  {
    for (int i = 0; i < 3; ++i) res[i] = 0;
    res[null_col] = 1;
  }
  else
  {
    res[2] = 1;
    for (int i = 0; i < 3; ++i)
      for (int j = i + 1; j < 3; ++j)
      {
        double det = c[i][0]*c[j][1] - c[i][1]*c[j][0];
        if (std::abs(det) > eps)
        {
          res[0] = (-c[i][2]*c[j][1] + c[j][2]*c[i][1]) / det;
          res[1] = (-c[i][0]*c[j][2] + c[j][0]*c[i][2]) / det;
        }
      }
  }

  for (int i = 0; i < 3; ++i) output << res[i] << " ";

  return 0;
}
