program galaxy_gen;
// Uses code by Tereguloff

{$APPTYPE CONSOLE}
{$R+}

uses SysUtils;

const
  max_coord = 1000;
  disp_max = 10;
  slim = 1000;

type
  TVector = class
  private
    items_: array of extended; 
    function GetItem(index: integer): extended;
    procedure SetItem(index: integer; x: extended);
  public
    property items[index: integer]: extended read GetItem write SetItem; default;
    constructor Create(n: integer);
    function Size: integer;
  end;
 
  TMatrix = class
  private 
    items_: array of array of extended;
    function GetItem(row, col: integer): extended;
    procedure SetItem(row, col: integer; x: extended);
  public
    property items[row, col: integer]: extended read GetItem write SetItem; default;
    constructor Create(rows, cols: integer);
    function RowCount: integer;
    function ColCount: integer;
  end;
  TVectorArray = array of TVector;
  TMatrixArray = array of TMatrix;
  TVectorArrayPtr = ^TVectorArray;
  TMatrixArrayPtr = ^TMatrixArray;

constructor TVector.Create (n: integer);
begin
  SetLength(items_, n);
end;
function TVector.Size: integer;
begin
  result := high(items_) + 1;
end;
function TVector.GetItem(index: integer): extended;
begin
  result := items_[index];
end;
procedure TVector.SetItem(index: integer; x: extended);
begin
  items_[index] := x;
end;

constructor TMatrix.Create(rows, cols: integer);
begin
  SetLength(items_, rows, cols);
end;
function TMatrix.RowCount: integer;
begin
  result := high(items_) + 1;
end;
function TMatrix.ColCount: integer;
begin
  result := high(items_[0]) + 1;
end;
function TMatrix.GetItem(row, col: integer): extended;
begin
  result := items_[row, col];
end;
procedure TMatrix.SetItem(row, col: integer; x: extended);
begin
  items_[row, col] := x;
end;

function GenerateNormalSelection (var mean, disp: TVector; var corr: TMatrix; count: integer): TMatrix;
var
  v1, v2, s, q: double;

  function getNextNormValue(Idx: integer): double;
  begin
    if (Idx mod 2 = 0) then begin
      repeat
        v1 := 2.0 * Random - 1.0;
        v2 := 2.0 * Random - 1.0;
        s := Sqr(v1) + Sqr(v2);
      until (1.0 - s > 1.0e-6);
      q := Sqrt(-2.0 * Ln(s) / s);
      Result := v1 * q;
    end
    else
      Result := v2 * q;
  end;

var
  a: array of array of double;
  etha: array of double;
  i, j, k, ai, aj, dimensions: integer;
  d: double;

begin
  dimensions := mean.Size;
  assert(dimensions = disp.Size);
  assert(dimensions = corr.RowCount); assert(dimensions = corr.ColCount);
  result := TMatrix.Create(count, dimensions);

  SetLength(a, Dimensions, Dimensions);
  for j := 1 to Dimensions do begin
    aj := j - 1;
    d := Corr[j-1, j-1]*disp[j-1];
    for k := 0 to aj - 1 do
      d := d - Sqr(a[aj, k]);
    d := Sqrt(d);
    for ai := 0 to aj - 1 do
      a[ai, aj] := 0.0;
    for i := j to Dimensions do begin
      ai := i - 1;
      a[ai, aj] := Corr[i-1, j-1]*sqrt(disp[i-1]*disp[j-1]);
      for k := 0 to aj - 1 do
        a[ai, aj] := a[ai, aj] - a[ai, k] * a[aj, k];
      a[ai, aj] := a[ai, aj] / d;
    end;
  end;
  SetLength(etha, Dimensions);
  try
    for i := 0 to Count - 1 do begin
      for j := 0 to Dimensions - 1 do
        etha[j] := getNextNormValue(j);
      for j := 0 to Dimensions - 1 do begin
        Result[i, j] := Mean[j {+ 1}];
        for k := 0 to j do
          Result[i, j] := Result[i, j] + etha[k] * a[j, k];
      end;
    end;
  except
    FreeAndNil(Result);
    raise;
  end;
end;

var 
  tempi, n, i, j, k, c1, c2, equal: integer;
  mean, disp, tv: TVector;
  cor, sel, r: TMatrix;
  t, alpha: double;

begin
//  assignfile (output, 'input.txt'); rewrite (output);

  val(paramstr(1), n, tempi);
  val(paramstr(2), randseed, tempi);
  val(paramstr(3), equal, tempi);

  mean := TVector.Create(3);
  disp := TVector.Create(3);
  cor := TMatrix.Create(3, 3);

  for i := 0 to 2 do
  begin
    mean[i] := 0;
    disp[i] := random(disp_max);
  end;

  t := disp[random(2)];
  repeat
    i := random(2); j := random(2);
  until i <> j;
  disp[i] := t; disp[j] := t;

  if equal = 1 then begin disp[1] := disp[0]; disp[2] := disp[0]; end;

  for i := 0 to 2 do 
  begin
    for j := 0 to 2 do 
      cor[i, j] := 0;
    cor[i,i] := 1;
  end;

  sel := GenerateNormalSelection(mean, disp, cor, n);

  r := TMatrix.Create(3, 3);
  tv := TVector.Create(3);

  for c1 := 0 to 2 do
    for c2 := c1 + 1 to 2 do 
    begin
      for i := 0 to 2 do 
        for j := 0 to 2 do
          r[i, j] := 0;

      alpha := 2*pi*random;
      r[c1, c1] := cos(alpha); r[c1, c2] := -sin(alpha);
      r[c2, c1] := sin(alpha); r[c2, c2] := cos(alpha);
      r[3 - c1 - c2, 3 - c1 - c2] := 1;
      for i := 0 to n-1 do
      begin
        for j := 0 to 2 do
        begin
          tv[j] := 0;
          for k := 0 to 2 do tv[j] := tv[j] + r[j, k]*sel[i, k];
        end;
        for j := 0 to 2 do sel[i, j] := tv[j];
      end;
    end;
   
  
  for i := 0 to 2 do
  begin
    mean[i] := random(max_coord) - max_coord / 2;
    for j := 0 to n-1 do 
      sel[j, i] := sel[j, i] + mean[i];
  end;
  
  writeln(n);
  for i := 0 to n-1 do
  begin
    for j := 0 to 2 do write(sel[i, j] : 0 : 4, ' ');
    writeln;
  end;

//  closefile (output);
end.
